#ifndef WMMSENS_CHIPSET_MGR_H
#define WMMSENS_CHIPSET_MGR_H

//
// chipset_mgr.h - new implementation of sensor manager classes
//   by Ed Porras <ep@bangchemistry.com>
//   with portions of code from
//
//     wmgtemp by Roger Dunce <kronos@nanunanu.org> 
//
//
// $Log: chipset_mgr.h,v $
// Revision 1.9  2003/03/25 02:14:24  ep
// moved cs_feature and chipset into their own files.. updated code
// to conform to gcc3 std
//
// Revision 1.8  2001/11/13 06:36:20  ep
// redeclared destructors as public.. minor sanity assignment
// for unkown f_type in switch..
//
// Revision 1.7  2001/11/12 08:10:54  ep
// reimplemented purge by using STL algorithm.. made
// destructor public..
//
// Revision 1.6  2001/11/12 02:53:26  ep
// added more sanity checks for updating mapped feature values..
// renamed checkLast to purgeLast.. made output more readable..
//
// Revision 1.5  2001/11/09 07:39:01  ep
// added api call to check if features should be ignored..
// removed ALARM type.. modified code so only features that have
// others mapped to it are maintained.. fixed stupid issue
// with xor vs 2's comp.. added warning for features that are
// mapped more than once
//
// Revision 1.4  2001/11/08 08:09:07  ep
// removed hoaky extension methods.. shouldn't be handled here
//
// Revision 1.3  2001/11/06 06:52:29  ep
// the beginnings of the better detect mechanism
//
// Revision 1.2  2001/11/05 08:04:16  ep
// implemented data retrieval method (uses single index)..
// added poll bit modes
//
// Revision 1.1  2001/11/05 05:24:26  ep
// Initial revision
//
// 

#include <iostream>
#include <vector>
#include <sensors/sensors.h>

#include "chipset.h"

namespace wmmsens
{
  //
  // maintains an arbitrary number of chipsets
  class chipset_mgr
  {
  private:
    // the features we're maintaining..
    std::vector<chipset *> chipset_list;
    ui32 total_features;
    ui8 history_steps;

    // prohibit
    chipset_mgr(const chipset_mgr&);
    chipset_mgr& operator=(const chipset_mgr&);

  public:
    // enables / disables messages
    static bool verbose;

    // constructor & destructor
    chipset_mgr(ui8 hist_steps) : 
      total_features(0), history_steps(hist_steps) { }
    ~chipset_mgr();

    // getters
    ui32 numFeatures() const { return total_features; } 

    // tries to detect chipsets with features that have standard
    // names.. poll_bits is a combination of chipset::csf_type's
    bool detect(ui16 poll_bits);

    // updates the readings of all registered sensors
    void update() {
      for (register ui32 i = 0; i < chipset_list.size(); i++)
	chipset_list[ i ]->update();
    }

    // fetches the feature at the index given
    const cs_feature& operator[](ui32 idx) const;

    friend std::ostream& operator<<(std::ostream &, const chipset_mgr&);

  private:
    // helper methods
    bool registerChipset(const sensors_chip_name& chipset, ui16 bits);
  };

} // namespace wmmsens

#endif
